(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["module", "exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports);
    global.undefined = mod.exports;
  }
})(this, function (module, exports) {
  "use strict";

  // TODO: Optimize using deferred parsing.
  // If a data URL string is passed in and the only operation called is
  // toString() then there is no need to parse and increase
  // memory consumption. This would complicate the code though
  // so only implement if needed.
  (function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
      var v = factory(require, exports);
      if (v !== undefined) module.exports = v;
    } else if (typeof define === "function" && define.amd) {
      define(["require", "exports", "./array-buffer-converter", "./media-type"], factory);
    }
  })(function (require, exports) {
    "use strict";

    Object.defineProperty(exports, "__esModule", {
      value: true
    }); // TODO: Refactor window dependencies on node
    // console.log(new Buffer('Hello World!').toString('base64'));
    // console.log(new Buffer(b64Encoded, 'base64').toString());
    // TODO: Refactor escape / unescape on node
    // https://nodejs.org/api/querystring.html

    const array_buffer_converter_1 = require("./array-buffer-converter");

    const media_type_1 = require("./media-type"); // data:[<MIME-type>][;charset=<encoding>][;base64],<data>
    // dataurl    := "data:" [ mediatype ] [ ";base64" ] "," data
    // mediatype  := [ type "/" subtype ] *( ";" parameter )
    // data       := *urlchar
    // parameter  := attribute "=" value


    class DataURL {
      get mediaType() {
        return this._mediaType;
      }

      set mediaType(mediaType) {
        if (mediaType instanceof media_type_1.default) {
          this._mediaType = mediaType;
        } else if (typeof mediaType === "string") {
          this._mediaType = new media_type_1.default(mediaType);
        } else {
          throw new Error("Media type must be 'string' or 'MediaType'");
        }
      }

      get isBase64() {
        return this._isBase64;
      }

      get data() {
        return this._data;
      }

      setBase64EncodedData(base64EncodedData) {
        this._isBase64 = true;
        this._data = base64EncodedData;
      }

      setURLEncodedData(urlEncodedData) {
        this._isBase64 = false;
        this._data = urlEncodedData;
      }

      constructor(data, options) {
        // Set default options
        if (!options) {
          options = {
            mediaType: null,
            encoding: "auto"
          };
        }

        if (!options.encoding) {
          options.encoding = "auto";
        } // Validate encoding


        if (options.encoding !== "auto" && options.encoding !== "url" && options.encoding !== "base64") {
          throw new Error("Unknown encoding (must be 'auto', 'url', or 'base64'): " + options.encoding);
        } // Save media type


        let mediaType = options.mediaType;

        if (typeof mediaType === "string") {
          mediaType = new media_type_1.default(mediaType);
        }

        this._mediaType = mediaType; // Save data and return if none

        if (!data) {
          this._isBase64 = false;
          this._data = data;
          return;
        } // Parse data


        if (data instanceof ArrayBuffer) {
          if (options.encoding === "base64" || options.encoding === "auto") {
            this.setBase64EncodedData(array_buffer_converter_1.default.toBase64(data));
          } else if (options.encoding === "url") {
            this.setURLEncodedData(encodeURIComponent(array_buffer_converter_1.default.toBinaryString(data)));
          }
        } else if (typeof data === "string") {
          // Ensure this is a data URI
          let startsWithData = data.slice(0, "data:".length) === "data:";

          if (!startsWithData) {
            throw new Error("Only 'data' URI strings are supported");
          } // Find the comma that separates the prefix from the data


          let commaIndex = data.indexOf(",");

          if (commaIndex === -1) {
            throw new Error("Missing comma in URL");
          } // Get prefix and data


          let prefix = data.slice(0, commaIndex);
          let encodedData = data.slice(commaIndex + 1); // Get is base64

          let prefixParts = prefix.split(";");
          let isBase64 = false;

          for (let i = 1; i < prefixParts.length; ++i) {
            let prefixPart = prefixParts[i].trim();

            if (prefixPart === "base64") {
              isBase64 = true;
              break;
            }
          } // Get media type


          mediaType = prefix.slice("data:".length, commaIndex);
          mediaType = mediaType.length === 0 ? null : mediaType;
          this._mediaType = this._mediaType || new media_type_1.default(mediaType); // Convert encoded data as needed

          if (options.encoding === "auto") {
            // Auto encoding saves the data URI as is
            this._isBase64 = isBase64;
            this._data = encodedData;
          } else if (options.encoding === "base64") {
            // Convert to base64
            this._isBase64 = true;

            if (isBase64) {
              this._data = encodedData;
            } else {
              this._data = window.btoa(unescape(encodedData));
            }
          } else if (options.encoding === "url") {
            // Convert to URL encoding
            this._isBase64 = false;

            if (!isBase64) {
              this._data = encodedData;
            } else {
              this._data = encodeURIComponent(window.atob(encodedData));
            }
          }
        } else {
          throw new Error("unsupported object type (must be ArrayBuffer or string): " + typeof data);
        }
      }

      toString() {
        return "data:" + (this._mediaType ? this._mediaType.toString() : "") + (this._isBase64 ? ";base64" : "") + "," + (this._data ? this._data : "");
      }

      toJSON() {
        return this.toString();
      }

      valueOf() {
        return this.toString();
      }

      toArrayBuffer() {
        if (!this._data) {
          return null;
        }

        if (this._isBase64) {
          return array_buffer_converter_1.default.fromBase64(this._data);
        } else {
          return array_buffer_converter_1.default.fromBinaryString(unescape(this._data));
        }
      }

      toBase64() {
        if (!this._data) {
          return this._data;
        }

        if (this._isBase64) {
          return this._data;
        } else {
          return window.btoa(unescape(this._data));
        }
      }

      toBinaryString() {
        if (!this._data) {
          return this._data;
        }

        if (this._isBase64) {
          return window.atob(this._data);
        } else {
          return unescape(this._data);
        }
      }

      toUnicodeString() {
        if (!this._data) {
          return this._data;
        }

        if (this._isBase64) {
          return decodeURIComponent(escape(window.atob(this._data)));
        } else {
          return decodeURIComponent(this._data);
        }
      }

      static createFromBase64(base64, options) {
        return new DataURL("data:;base64," + base64, options);
      }

      static createFromBinaryString(binary, options) {
        if (!options) {
          options = {
            encoding: "auto"
          };
        }

        if (options.encoding === "base64" || options.encoding === "auto") {
          return new DataURL("data:;base64," + window.btoa(binary), options);
        } else if (options.encoding === "url") {
          return new DataURL("data:," + encodeURIComponent(binary), options);
        }
      }

      static createFromUnicodeString(text, options) {
        return new DataURL("data:," + encodeURIComponent(text), options);
      }

    }

    exports.default = DataURL;
  });
});
//# sourceMappingURL=data-url.js.map
