(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["module", "exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(module, exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod, mod.exports);
    global.undefined = mod.exports;
  }
})(this, function (module, exports) {
  "use strict";

  /**
   * media-type
   * @author Lovell Fuller (original JS)
   * @author Aaron Oneal (TypeScript)
   *
   * This code is distributed under the Apache License Version 2.0, the terms of
   * which may be found at http://www.apache.org/licenses/LICENSE-2.0.html
   */
  (function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
      var v = factory(require, exports);
      if (v !== undefined) module.exports = v;
    } else if (typeof define === "function" && define.amd) {
      define(["require", "exports"], factory);
    }
  })(function (require, exports) {
    "use strict";

    Object.defineProperty(exports, "__esModule", {
      value: true
    });
    const mediaTypeMatcher = /^(application|audio|image|message|model|multipart|text|video)\/([a-zA-Z0-9!#$%^&\*_\-\+{}\|'.`~]{1,127})(;.*)?$/;
    const parameterSplitter = /;(?=(?:[^\"]*\"[^\"]*\")*(?![^\"]*\"))/;

    class MediaType {
      constructor(mediaType) {
        this._type = null;
        this.setSubtypeAndSuffix(null);
        this._parameters = {};

        if (mediaType) {
          let match = mediaType.match(mediaTypeMatcher);

          if (match) {
            this._type = match[1];
            this.setSubtypeAndSuffix(match[2]);

            if (match[3]) {
              match[3].substr(1).split(parameterSplitter).forEach(parameter => {
                let keyAndValue = parameter.split("=", 2);

                if (keyAndValue.length === 2) {
                  this._parameters[keyAndValue[0].toLowerCase().trim()] = this.unwrapQuotes(keyAndValue[1].trim());
                }
              });
            }
          }
        }
      }

      get type() {
        return this._type;
      }

      set type(type) {
        this._type = type;
      }

      get subtype() {
        return this._subtype;
      }

      set subtype(subtype) {
        this.setSubtypeAndSuffix(subtype);
      }

      get parameters() {
        return this._parameters;
      }

      set parameters(parameters) {
        this._parameters = parameters;
      }

      get isValid() {
        return this._type !== null && this._subtype !== null && this._subtype !== "example";
      }

      get hasSuffix() {
        return !!this._suffix;
      }

      get isVendor() {
        return this.firstSubtypeFacetEquals("vnd");
      }

      get isPersonal() {
        return this.firstSubtypeFacetEquals("prs");
      }

      get isExperimental() {
        return this.firstSubtypeFacetEquals("x") || this._subtype.substring(0, 2).toLowerCase() === "x-";
      }

      toString() {
        let str = "";

        if (this.isValid) {
          str = str + this._type + "/" + this._subtype;

          if (this.hasSuffix) {
            str = str + "+" + this._suffix;
          }

          let parameterKeys = Object.keys(this._parameters);

          if (parameterKeys.length > 0) {
            let parameters = [];
            let that = this;
            parameterKeys.sort((a, b) => {
              return a.localeCompare(b);
            }).forEach(element => {
              parameters.push(element + "=" + this.wrapQuotes(that._parameters[element]));
            });
            str = str + ";" + parameters.join(";");
          }
        }

        return str;
      }

      toJSON() {
        return this.toString();
      }

      valueOf() {
        return this.toString();
      }

      setSubtypeAndSuffix(subtype) {
        this._subtype = subtype;
        this._subtypeFacets = [];
        this._suffix = null;

        if (subtype) {
          if (subtype.indexOf("+") > -1 && subtype.substr(-1) !== "+") {
            let fixes = subtype.split("+", 2);
            this._subtype = fixes[0];
            this._subtypeFacets = fixes[0].split(".");
            this._suffix = fixes[1];
          } else {
            this._subtypeFacets = subtype.split(".");
          }
        }
      }

      firstSubtypeFacetEquals(str) {
        return this._subtypeFacets.length > 0 && this._subtypeFacets[0] === str;
      }

      wrapQuotes(str) {
        return str.indexOf(";") > -1 ? "\"" + str + "\"" : str;
      }

      unwrapQuotes(str) {
        return str.substr(0, 1) === "\"" && str.substr(-1) === "\"" ? str.substr(1, str.length - 2) : str;
      }

    }

    exports.default = MediaType;
  });
});
//# sourceMappingURL=media-type.js.map
